import QtQuick 2.12
import QtQuick.Controls 2.5
import "../Sending"
import "../Receiving"
import "../Messages"
import "../Settings"
import "../Help"
import "../Connect"
import "../Notifications"

ApplicationWindow {
    id: window
    visible: true
    width: 1280
    height: 720
    maximumHeight: height
    maximumWidth: width
    font.capitalization: Font.MixedCase

    minimumHeight: height
    minimumWidth: width
    title: qsTr("Covert Channel - Control Panel")

    property string notificationTabTitle: "Notifications"

    Connections {
        target: _qmlHandler

        onNotification: {
            if (swipeView.currentIndex != 5)
              notificationTabTitle = "* Notifications"
        }
    }

    SwipeView {
        id: swipeView
        anchors.fill: parent
        currentIndex: header.currentIndex

        SendingForm {

        }

        ReceivingForm {

        }

        MessagesForm {

        }

        SettingsForm {

        }

        HelpForm {

        }

        NotificationsForm {

        }
    }

    header: TabBar {
        id: header
        currentIndex: swipeView.currentIndex
        contentHeight: 50

        TabButton {
            text: qsTr("Sending")
        }

        TabButton {
            text: qsTr("Receiving")
        }

        TabButton {
            text: qsTr("Messages")
        }

        TabButton {
            text: qsTr("Settings")
        }

        TabButton {
            text: qsTr("Help")
        }

        TabButton {
            text: notificationTabTitle

            onClicked: {
                text = "Notifications"
            }
        }
    }

    footer: FooterForm {

    }

    LoginSignupPopup {
        id: loginSignupPopup
    }

    IpPopup {
        id: ipPopup
    }

    NoConfigFoundPopup {
        id: noConfigFoundPopup
    }

    InvalidConfigPopup {
        id: invalidConfigPopup
    }

    InvalidCliPathPopup {
        id: invalidCliPathPopup
    }

    Component.onCompleted: {
      swipeView.interactive = false
      ipPopup.open()
      _qmlHandler.onStart()
    }
}
